Crate insectbox

Crate insectbox 

InsectBox

This crate aims to provide in-memory security inspired by OpenSSH in-memory key encryption.

Warning

This crate is very unsafe and full of unsafe code. It should not be used for any real aplication, UNLESS it actually is deemed safe enough after reviewing the source code.

Features

FeatureDescription
secboxSecBox<T> and InsecBox<T> types
cryptboxCryptBox<T> and PlainBox<T> types

Note: the secbox feature is enabled by default

The utils module

This module is public for one simple reason: to allow other users to implement types similar or better than the ones provided in this crate. It also provides almost the same utilities as the memsec crate, but actually documents them.

Modules

utils
Memory management utilities

Structs

CryptBox
An “encrypted” box type, which encrypts the inner data
InsecBox
An unprotected pointer to the data inside a SecBox<T>
PlainBox
A pointer to the decrypted data inside a CryptBox<T>
SecBox
A “secure” box type, which shields the inner data from buffer overreads